"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bind = exports.func = void 0;
var mimic_fn_1 = __importDefault(require("mimic-fn"));
var partial_func_1 = require("../server/partial-func");
var rpc_1 = require("../server/rpc");
var createRemoteFunc = function (source) {
    var remoteFunction = function () {
        throw new Error('Unbound remote function');
    };
    remoteFunction.isRemoteFunction = true;
    remoteFunction.source = source;
    remoteFunction.bound = false;
    return remoteFunction;
};
/** Bind a service to a client in such way that the service can be used outside of a query function through RPC */
var bindServiceRpc = function (client, serviceName) {
    var binds = {};
    return new Proxy({}, {
        get: function (target, prop, receiver) {
            if (!binds.hasOwnProperty(prop)) {
                // binds[prop] = bind(client, func(`async (...args) => ${serviceName}.${prop}(...args)`))
                binds[prop] = bind(client, (0, exports.func)((0, rpc_1.createRpcCommand)(serviceName, prop)));
            }
            return binds[prop];
        }
    });
};
/** Create a new remote function */
var func = function (sourceInput) {
    if (typeof sourceInput === 'string') {
        sourceInput = sourceInput;
    }
    else if (Array.isArray(sourceInput)) {
        sourceInput = sourceInput.join('');
    }
    else if (typeof sourceInput === 'function') {
        throw new Error("wrong use of 'func', use the bundled babel plugin");
    }
    else {
        throw new Error("wrong use of 'func', unsupported source type");
    }
    return createRemoteFunc(sourceInput);
};
exports.func = func;
function bind(client, target) {
    if (!client) {
        throw new Error('Invalid client');
    }
    if (typeof target === 'string') {
        return bindServiceRpc(client, target);
    }
    if (!target || !target.isRemoteFunction) {
        throw new Error('Invalid remote function');
    }
    if (target.bound) {
        throw new Error('Remote function already bound');
    }
    var source = target.source;
    var boundRemoteFunction = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _args = (args || []).map(function (arg) {
            if (typeof arg === 'function' && arg.isRemoteFunction) {
                return (0, partial_func_1.createPartialFunc)(arg.source);
            }
            return arg;
        });
        return client.request(source, _args);
    };
    (0, mimic_fn_1.default)(boundRemoteFunction, target);
    boundRemoteFunction.target = target;
    return boundRemoteFunction;
}
exports.bind = bind;
//# sourceMappingURL=func.js.map